/*
 * Copyright © 2021 Fachhochschule für die Wirtschaft (FHDW) Hannover
 *
 * This file is part of ipspiel22-tictactoe-core.
 *
 * ipspiel22-tictactoe-core is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * ipspiel22-tictactoe-core is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * ipspiel22-tictactoe-core. If not, see <http://www.gnu.org/licenses/>.
 */
package de.fhdw.gaming.ipspiel22.tictactoe.core.domain;

import java.util.List;
import java.util.Optional;

/**
 * Represents a complete row on a {@link TicTacToeBoard board}.
 */
public interface TicTacToeRow {

    /**
     * Returns the type of this row.
     */
    TicTacToeRowType getType();

    /**
     * Returns the index of this row. For horizontal or vertical rows, its range extends from zero to the number of
     * rows/columns minus one. For diagonal rows, only zero (main diagonal) or one (secondary diagonal) are possible.
     */
    int getIndex();

    /**
     * Returns a row state if all fields in this row possess the same field state. Note that this also includes the
     * state {@link TicTacToeFieldState#EMPTY}.
     */
    Optional<TicTacToeFieldState> getState();

    /**
     * Returns the fields of this row.
     */
    List<? extends TicTacToeField> getFields();
}
