/*
 * Copyright © 2021 Fachhochschule für die Wirtschaft (FHDW) Hannover
 *
 * This file is part of ipspiel22-tictactoe-core.
 *
 * ipspiel22-tictactoe-core is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * ipspiel22-tictactoe-core is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * ipspiel22-tictactoe-core. If not, see <http://www.gnu.org/licenses/>.
 */
package de.fhdw.gaming.ipspiel22.tictactoe.core.domain.impl;

import java.util.List;
import java.util.Objects;
import java.util.Optional;

import de.fhdw.gaming.ipspiel22.tictactoe.core.domain.TicTacToeField;
import de.fhdw.gaming.ipspiel22.tictactoe.core.domain.TicTacToeFieldState;
import de.fhdw.gaming.ipspiel22.tictactoe.core.domain.TicTacToeRow;
import de.fhdw.gaming.ipspiel22.tictactoe.core.domain.TicTacToeRowType;

/**
 * Implements {@link TicTacToeRow}.
 */
public final class TicTacToeRowImpl implements TicTacToeRow {

    /**
     * The type of this row.
     */
    private final TicTacToeRowType type;
    /**
     * The zero-based index of this row.
     */
    private final int index;
    /**
     * The fields of this row.
     */
    private final List<? extends TicTacToeField> fields;

    /**
     * Creates a row.
     *
     * @param type   The type of this row.
     * @param index  The zero-based index of this row.
     * @param fields The fields of this row.
     */
    public TicTacToeRowImpl(final TicTacToeRowType type, final int index, final List<? extends TicTacToeField> fields) {
        this.type = type;
        this.index = index;
        this.fields = fields;
    }

    @Override
    public boolean equals(final Object obj) {
        if (obj instanceof TicTacToeRowImpl) {
            final TicTacToeRowImpl other = (TicTacToeRowImpl) obj;
            return this.type.equals(other.type) && this.index == other.index && this.fields.equals(other.fields);
        }
        return false;
    }

    @Override
    public int hashCode() {
        return Objects.hash(this.type, this.index, this.fields);
    }

    @Override
    public String toString() {
        return String.format("%s@%d%s", this.type, this.index, this.fields);
    }

    @Override
    public TicTacToeRowType getType() {
        return this.type;
    }

    @Override
    public int getIndex() {
        return this.index;
    }

    @Override
    public List<? extends TicTacToeField> getFields() {
        return this.fields;
    }

    @Override
    public Optional<TicTacToeFieldState> getState() {
        TicTacToeFieldState lastState = null;
        for (final TicTacToeField field : this.fields) {
            if (lastState == null) {
                lastState = field.getState();
            } else if (!lastState.equals(field.getState())) {
                return Optional.empty();
            }
        }
        return Optional.ofNullable(lastState); // may be empty if row is
    }
}
